from statistics import mean
from typing import List

from begroing_index.exceptions import UncertainIndexException


def aip_calc_ref(calc: float, toc: float):
    if calc < 1.0 and toc > 2.0:
        return 6.02
    if calc <= 1.0 and toc < 2.0:
        return 6.52
    if 4.0 > calc >= 1.0:
        return 6.86
    if calc >= 4.0:
        return 7.1


def calc_aip(indicator_values: List[float]):
    if len(indicator_values) < 3:
        raise UncertainIndexException(
            "Unable to calculate index due to too few indicator species"
        )

    return mean(indicator_values)


def calc_aip_eqr(
    aip: float,
    min_aip: float,
    calc: float,
    toc: float,
):
    ref_value = aip_calc_ref(calc=calc, toc=toc)
    return (aip - min_aip) / (ref_value - min_aip)



import csv
import os
from statistics import mean
from typing import Dict

iv_files = {
    "PIT": "indicator_values_PIT.csv",
    "AIP": "indicator_values_AIP.csv",
}


def load_indicator_values(index_name: str) -> Dict[str, float]:
    indicator_values = {}
    with open(
        os.path.join(
            os.path.dirname(__file__), "../indicator_values", iv_files[index_name]
        )
    ) as f:
        reader = csv.DictReader(f, delimiter=";")

        for row in reader:
            if len(row.values()) != 2:
                raise Exception(f"Could not read csv line: {row}")
            iv = float(row["indicator_value"].strip())
            assert iv > 0
            latin_name = row["latin_name"].strip()
            assert len(latin_name) > 0
            indicator_values[latin_name] = iv
    return indicator_values


def calc_max_iv(indicator_values: Dict[str, float], min_species_count: int) -> float:
    sorted_iv = sorted(indicator_values.values(), reverse=True)
    top_values = sorted_iv[:min_species_count]
    return mean(top_values)


def calc_min_iv(indicator_values: Dict[str, float], min_species_count: int) -> float:
    sorted_iv = sorted(indicator_values.values())
    bottom_values = sorted_iv[:min_species_count]
    return mean(bottom_values)
